#include <windows.h>
#include <stdio.h>


// Define CHUNKHDR
typedef struct CHUNKHDR 
{
  FOURCC ckid;		// chunk ID
  DWORD dwSize; 	// chunk size
} CHUNKHDR;


// Simplified header for standard WAV files
typedef struct PCMWAVEHDR_TAG 
{
  CHUNKHDR		chkRiff;
  FOURCC		fccWave;
  CHUNKHDR		chkFmt;
  PCMWAVEFORMAT pwf;
  CHUNKHDR		chkData;
} PCMWAVEHDR;


#define FOURCC_WAVE 0x45564157
#define FOURCC_FMT  0x20746D66
#define FOURCC_DATA 0x61746164

__declspec(dllexport)  int outputHeaderWavEncode( FILE *out , DWORD length, DWORD rate_used)
{
	PCMWAVEHDR	waveHdr;

	// Fill header parameters
	waveHdr.chkRiff.ckid			= FOURCC_RIFF;
	waveHdr.chkRiff.dwSize			= length + sizeof(PCMWAVEHDR) - sizeof(CHUNKHDR) ;
	waveHdr.fccWave					= FOURCC_WAVE;
	waveHdr.chkFmt.ckid				= FOURCC_FMT;
	waveHdr.chkFmt.dwSize			= sizeof (PCMWAVEFORMAT);
	waveHdr.pwf.wf.wFormatTag		= WAVE_FORMAT_PCM;
	waveHdr.pwf.wf.nChannels		= 2;
	if (rate_used == 0)				waveHdr.pwf.wf.nSamplesPerSec = 48000;
		else if (rate_used == 1)	waveHdr.pwf.wf.nSamplesPerSec = 44100;
		else if (rate_used == 2)	waveHdr.pwf.wf.nSamplesPerSec = 32000;
		else						waveHdr.pwf.wf.nSamplesPerSec = 48000;
	waveHdr.pwf.wBitsPerSample		= 16;
	waveHdr.pwf.wf.nBlockAlign		= waveHdr.pwf.wf.nChannels * ((waveHdr.pwf.wBitsPerSample + 7) / 8);
	waveHdr.pwf.wf.nAvgBytesPerSec	= waveHdr.pwf.wf.nBlockAlign * waveHdr.pwf.wf.nSamplesPerSec;
	waveHdr.chkData.ckid			= FOURCC_DATA;
	waveHdr.chkData.dwSize			= length;

	// Seek to beginning of file
	fseek(out, 0L, SEEK_SET);

	// And write to the audio file
	return(fwrite (&waveHdr, sizeof (waveHdr), 1, out));
}


__declspec(dllexport)  int closeWavEncode(FILE* out)
{
return(fclose(out));
}

__declspec(dllexport)  FILE * openWavEncode(char * file_buffer)
{
return(fopen(file_buffer, "w+b" ));
}

__declspec(dllexport)  int outputWavEncode(char * buffer, int length, FILE * out)
{
return(fwrite(buffer, 1, length, out)); 
}

